@@ -36,7 +36,9 @@
 
 /* {{{ globals */
 static char *xc_coredump_dir = NULL;
-static zend_bool xc_coredump_type = 0;
+#ifdef ZEND_WIN32
+static zend_ulong xc_coredump_type = 0;
+#endif
 static zend_bool xc_disable_on_crash = 0;
 
 static zend_compile_file_t *old_compile_file = NULL;
@@ -54,6 +56,21 @@ static zend_op_array *xc_check_initial_compile_file(zend_file_handle *h, int typ
 }
 /* }}} */
 
+/* devel helper function */
+#if 0
+static void xc_list_extensions() /* {{{ */
+{
+	zend_llist_element *element;
+	zend_extension *ext;
+	fprintf(stderr, "extensions:\n");
+	for (element = zend_extensions.head; element; element = element->next) {
+		ext = (zend_extension *) element->data;
+		fprintf(stderr, " - %s\n", ext->name);
+	}
+}
+/* }}} */
+#endif
+
 /* module helper function */
 static int xc_init_constant(int module_number TSRMLS_DC) /* {{{ */
 {
@@ -90,15 +107,17 @@ static int xc_init_constant(int module_number TSRMLS_DC) /* {{{ */
 		}
 	}
 
-	zend_register_long_constant(ZEND_STRS("XC_SIZEOF_TEMP_VARIABLE"), sizeof(temp_variable), CONST_CS | CONST_PERSISTENT, module_number TSRMLS_CC);
-	zend_register_stringl_constant(ZEND_STRS("XCACHE_VERSION"), ZEND_STRL(XCACHE_VERSION), CONST_CS | CONST_PERSISTENT, module_number TSRMLS_CC);
-	zend_register_stringl_constant(ZEND_STRS("XCACHE_MODULES"), ZEND_STRL(XCACHE_MODULES), CONST_CS | CONST_PERSISTENT, module_number TSRMLS_CC);
+	zend_register_long_constant(XCACHE_STRS("XC_SIZEOF_TEMP_VARIABLE"), sizeof(temp_variable), CONST_CS | CONST_PERSISTENT, module_number TSRMLS_CC);
+	zend_register_stringl_constant(XCACHE_STRS("XCACHE_VERSION"), XCACHE_STRL(XCACHE_VERSION), CONST_CS | CONST_PERSISTENT, module_number TSRMLS_CC);
+	zend_register_stringl_constant(XCACHE_STRS("XCACHE_MODULES"), XCACHE_STRL(XCACHE_MODULES), CONST_CS | CONST_PERSISTENT, module_number TSRMLS_CC);
 	return 0;
 }
 /* }}} */
 /* {{{ PHP_GINIT_FUNCTION(xcache) */
+#ifdef __GNUC__
 #pragma GCC push_options
 #pragma GCC diagnostic ignored "-Wshadow"
+#endif
 
 #ifdef PHP_GINIT_FUNCTION
 static PHP_GINIT_FUNCTION(xcache)
@@ -106,7 +125,9 @@ static PHP_GINIT_FUNCTION(xcache)
 static void xc_init_globals(zend_xcache_globals* xcache_globals TSRMLS_DC)
 #endif
 {
+#ifdef __GNUC__
 #pragma GCC pop_options
+#endif
 
 	memset(xcache_globals, 0, sizeof(zend_xcache_globals));
 
@@ -190,6 +211,7 @@ PHP_FUNCTION(xcache_get_isref)
 #ifdef HAVE_XCACHE_DPRINT
 /* {{{ proto bool  xcache_dprint(mixed value)
    Prints variable (or value) internal struct (debug only) */
+#include "xc_processor.h"
 PHP_FUNCTION(xcache_dprint)
 {
 	zval *value;
@@ -294,10 +316,10 @@ PHP_FUNCTION(xcache_get_opcode_spec)
 		opspec = xc_get_opcode_spec((zend_uchar) spec);
 		if (opspec) {
 			array_init(return_value);
-			add_assoc_long_ex(return_value, ZEND_STRS("ext"), opspec->ext);
-			add_assoc_long_ex(return_value, ZEND_STRS("op1"), opspec->op1);
-			add_assoc_long_ex(return_value, ZEND_STRS("op2"), opspec->op2);
-			add_assoc_long_ex(return_value, ZEND_STRS("res"), opspec->res);
+			add_assoc_long_ex(return_value, XCACHE_STRS("ext"), opspec->ext);
+			add_assoc_long_ex(return_value, XCACHE_STRS("op1"), opspec->op1);
+			add_assoc_long_ex(return_value, XCACHE_STRS("op2"), opspec->op2);
+			add_assoc_long_ex(return_value, XCACHE_STRS("res"), opspec->res);
 			return;
 		}
 	}
@@ -328,7 +350,14 @@ PHP_FUNCTION(xcache_get_special_value)
 		break;
 
 	default:
-		RETURN_NULL();
+		if ((Z_TYPE_P(value) & ~IS_CONSTANT_TYPE_MASK)) {
+			*return_value = *value;
+			zval_copy_ctor(return_value);
+			return_value->type &= IS_CONSTANT_TYPE_MASK;
+		}
+		else {
+			RETURN_NULL();
+		}
 	}
 }
 /* }}} */
@@ -564,10 +593,10 @@ typedef struct {
 	startup_func_t old_startup;
 } xc_incompatible_zend_extension_info_t;
 static xc_incompatible_zend_extension_info_t xc_incompatible_zend_extensions[] = {
+	{ "Zend Extension Manager", NULL },
 	{ "Zend Optimizer", NULL },
 	{ "the ionCube PHP Loader", NULL }
 };
-zend_llist_element **xc_zend_extension_elements;
 
 static xc_incompatible_zend_extension_info_t *xc_get_incompatible_zend_extension_info(const char *name)
 {
@@ -600,23 +629,25 @@ static int xc_incompatible_zend_extension_startup_hook(zend_extension *extension
 	xc_incompatible_zend_extension_info_t *incompatible_zend_extension_info = xc_get_incompatible_zend_extension_info(extension->name);
 	int status;
 	zend_bool catched = 0;
-	zend_llist old_zend_extensions;
+	zend_llist saved_zend_extensions_container; /* without elements */
+	zend_llist_element **saved_zend_extensions_elments;
+	size_t new_zend_extensions_elments_count;
+	zend_llist_element **new_zend_extensions_elments;
 	zend_extension *ext;
 	size_t i;
 	zend_llist_element *element;
 	TSRMLS_FETCH();
 
-	/* restore */
+	/* restore startup hack */
 	extension->startup = incompatible_zend_extension_info->old_startup;
 	incompatible_zend_extension_info->old_startup = NULL;
 	assert(extension->startup);
 
-	/* save */
-	assert(!xc_zend_extension_elements);
-	old_zend_extensions = zend_extensions;
-	xc_zend_extension_elements = malloc(sizeof(zend_llist_element *) * old_zend_extensions.count);
-	for (i = 0, element = old_zend_extensions.head; element; ++i, element = element->next) {
-		xc_zend_extension_elements[i] = element;
+	/* save extensions list */
+	saved_zend_extensions_container = zend_extensions;
+	saved_zend_extensions_elments = malloc(sizeof(zend_llist_element *) * saved_zend_extensions_container.count);
+	for (i = 0, element = saved_zend_extensions_container.head; element; ++i, element = element->next) {
+		saved_zend_extensions_elments[i] = element;
 	}
 
 	/* hide all XCache extensions from it */
@@ -624,8 +655,8 @@ static int xc_incompatible_zend_extension_startup_hook(zend_extension *extension
 	zend_extensions.tail = NULL;
 	zend_extensions.count = 0;
 
-	for (i = 0; i < old_zend_extensions.count; ++i) {
-		element = xc_zend_extension_elements[i];
+	for (i = 0; i < saved_zend_extensions_container.count; ++i) {
+		element = saved_zend_extensions_elments[i];
 		element->next = element->prev = NULL;
 
 		ext = (zend_extension *) element->data;
@@ -643,21 +674,46 @@ static int xc_incompatible_zend_extension_startup_hook(zend_extension *extension
 		catched = 1;
 	} zend_end_try();
 
-	/* restore */
-	zend_extensions = old_zend_extensions;
+	/* save newly added extensions added by this extension*/
+	new_zend_extensions_elments_count = zend_extensions.count - 1;
+	new_zend_extensions_elments = NULL;
+	if (new_zend_extensions_elments_count) {
+		new_zend_extensions_elments = malloc(sizeof(zend_llist_element *) * new_zend_extensions_elments_count);
+		element = zend_extensions.head;
+		for (i = 0, element = element->next; element; ++i, element = element->next) {
+			new_zend_extensions_elments[i] = element;
+		}
+	}
+
+	/* restore original extension list*/
+	zend_extensions = saved_zend_extensions_container;
 	zend_extensions.head = NULL;
 	zend_extensions.tail = NULL;
 	zend_extensions.count = 0;
-	for (i = 0; i < old_zend_extensions.count; ++i) {
-		element = xc_zend_extension_elements[i];
+	for (i = 0; i < saved_zend_extensions_container.count; ++i) {
+		element = saved_zend_extensions_elments[i];
 		element->next = element->prev = NULL;
 
 		xc_zend_llist_add_element(&zend_extensions, element);
 		++zend_extensions.count;
-	}
 
-	free(xc_zend_extension_elements);
-	xc_zend_extension_elements = NULL;
+		ext = (zend_extension *) element->data;
+		if (ext == extension && new_zend_extensions_elments_count) {
+			/* add new created extension */
+			size_t j;
+			for (j = 0; j < new_zend_extensions_elments_count; ++j) {
+				element = new_zend_extensions_elments[j];
+				element->next = element->prev = NULL;
+
+				xc_zend_llist_add_element(&zend_extensions, element);
+				++zend_extensions.count;
+			}
+		}
+	}
+	free(saved_zend_extensions_elments);
+	if (new_zend_extensions_elments) {
+		free(new_zend_extensions_elments);
+	}
 
 	if (catched) {
 		zend_bailout();
@@ -720,7 +776,9 @@ static zend_extension xc_zend_extension_entry = {
 /* {{{ PHP_INI */
 PHP_INI_BEGIN()
 	PHP_INI_ENTRY1     ("xcache.coredump_directory",      "", PHP_INI_SYSTEM, xcache_OnUpdateString,   &xc_coredump_dir)
+#ifdef ZEND_WIN32
 	PHP_INI_ENTRY1     ("xcache.coredump_type",          "0", PHP_INI_SYSTEM, xcache_OnUpdateULong,    &xc_coredump_type)
+#endif
 	PHP_INI_ENTRY1_EX  ("xcache.disable_on_crash",       "0", PHP_INI_SYSTEM, xcache_OnUpdateBool,     &xc_disable_on_crash, zend_ini_boolean_displayer_cb)
 	PHP_INI_ENTRY1_EX  ("xcache.test",                   "0", PHP_INI_SYSTEM, xcache_OnUpdateBool,     &xc_test,             zend_ini_boolean_displayer_cb)
 	STD_PHP_INI_BOOLEAN("xcache.experimental",           "0", PHP_INI_ALL,    OnUpdateBool,        experimental,      zend_xcache_globals, xcache_globals)
