@@ -2,7 +2,7 @@
 #define __XCACHE_H
 #define XCACHE_NAME       "XCache"
 #ifndef XCACHE_VERSION
-#	define XCACHE_VERSION "3.0.1"
+#	define XCACHE_VERSION "3.1.0"
 #endif
 #define XCACHE_AUTHOR     "mOo"
 #define XCACHE_COPYRIGHT  "Copyright (c) 2005-2013"
@@ -19,7 +19,7 @@
 #include <config.h>
 #endif
 #include "xcache/xc_shm.h"
-#include "xcache/xc_lock.h"
+#include "xcache/xc_mutex.h"
 #include "xcache/xc_compatibility.h"
 
 extern zend_module_entry xcache_module_entry;
