@@ -9,6 +9,10 @@
 #else
 #	define OPSPEC_VAR_2 OPSPEC_VAR
 #endif
+#ifdef ZEND_ENGINE_2_4
+#undef OPSPEC_FETCH
+#define OPSPEC_FETCH OPSPEC_STD
+#endif
 #include "xc_opcode_spec_def.h"
 
 zend_uchar xc_get_opcode_spec_count()
