@@ -13,6 +13,9 @@ ZEND_BEGIN_MODULE_GLOBALS(xcache)
 	zend_bool coverager_autostart;
 	zend_bool coverager_started;
 	HashTable *coverages;  /* coverages[file][line] = times */
+#endif
+#ifndef ZEND_WIN32
+	pid_t holds_pid;
 #endif
 	xc_stack_t *php_holds;
 	zend_uint php_holds_size;
@@ -40,6 +43,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xcache)
 	zend_bool internal_table_copied;
 
 	void *sandbox;
+	zend_uint op_array_dummy_refcount_holder;
 ZEND_END_MODULE_GLOBALS(xcache)
 
 ZEND_EXTERN_MODULE_GLOBALS(xcache)
